const OracleService = require('../service/OracleService');

class updaterProdutosModel 
{  
  getUpdaterProdutos(isActive = false) {
    const WHERE_CLAUSE = isActive ? "WHERE IDATIVO = 'S'" : '';
    let sql = `
      SELECT 
        ID, 
        NMPRODUCT, 
        IDUTMULTVER
      FROM UPDATERPRODUTOS
      ${WHERE_CLAUSE}
      ORDER BY NMPRODUCT
    `;
    return OracleService.fetchAssoc(sql);
  }

  getUpdaterProdutosPerCasherFilial(id) {
    let sql = `
    SELECT  F.CDFILIAL, F.NMFILIAL, F.CDFILIAL || ' - ' || F.NMFILIAL AS CDFILIAL_NMFILIAL
      FROM FILIAL F
    INNER JOIN UPDATERPRODFILI UPF ON UPF.CDFILIAL = F.CDFILIAL
      WHERE UPF.UPDATERPRODID = :PRODUCT_ID
    ORDER BY F.CDFILIAL    
    `; 
    const params = {
      PRODUCT_ID: id,
    }        
    return OracleService.fetchAssoc(sql, params);
  }

  getUpdaterProdutosById(id) {
    const sql = `
      SELECT ID, NMPRODUCT, IDUTMULTVER
        FROM UPDATERPRODUTOS
       WHERE ID = :ID
    `;

    const params = {
      ID: id,
    }
    
    return OracleService.fetchAssoc(sql, params);
  }

  getUpdaterProdutosWithUnits(id) {
    const sql = `
      SELECT
        PF.ID AS PRODFILIID,
        PF.CDFILIAL || ' - ' || F.NMFILIAL AS FILIAL,
        PF.CDFILIAL,
        F.NMFILIAL,
        E.SGESTADO,
        P.NMPRODUCT,
        P.IDUTMULTVER,
        P.ID,
        DECODE(PF.NRVERSAOATUAL, NULL, 'S', 'N') AS IDINSTPEND,
        CASE
            WHEN PF.NRVERSAOATUAL IS NULL AND EV.IDSTEVENTO IS NULL THEN
                'INSTALACAO_PENDENTE'
            WHEN EV.IDSTEVENTO IS NULL THEN
                'INSTALADO'
            ELSE
                EV.IDSTEVENTO
        END AS IDSTEVENTO,
        DECODE(PF.NRVERSAOATUAL, NULL, '-', PF.NRVERSAOATUAL) AS NRVERSAOATUAL
      FROM
        UPDATERPRODUTOS P
        JOIN UPDATERPRODFILI PF ON (PF.UPDATERPRODID = P.ID)
        LEFT JOIN FILIAL F ON (PF.CDFILIAL = F.CDFILIAL)
        LEFT JOIN ENDEFILI E ON (E.CDFILIAL = F.CDFILIAL AND E.IDTPENDEFILI = 'P')
        LEFT JOIN UPDATEREVENTOS  EV ON (EV.CDPRODUCTID = P.ID AND EV.CDPRODFILIID = PF.ID AND EV.CDFILIAL = PF.CDFILIAL AND EV.IDSTEVENTO NOT IN ( 'FINALIZADO', 'CANCELADO' ))
      WHERE
        P.ID = :ID
      ORDER BY
        F.CDFILIAL ASC,
        PF.ID ASC
    `;

    const params = {
      ID: id,
    }
    
    return OracleService.fetchAssoc(sql, params);
  }

  getUpdaterProdutosWithUnitsPerCasher({ PRODUCT_ID, CDFILIAL }) {
    const sql = `
    SELECT
      UCX.ID AS PRODFILIID,
      F.CDFILIAL,
      F.NMFILIAL,
      F.CDFILIAL || ' - ' || F.NMFILIAL AS FILIAL,
      E.SGESTADO,
      C.CDCAIXA,
      C.NMCAIXA,
      C.CDCAIXA || ' - ' || C.NMCAIXA AS CDCAIXA_NMCAIXA,
      UP.ID AS CDPRODUCTID,
      UP.NMPRODUCT,
      UP.IDUTMULTVER,
      DECODE(UCX.NRVERSAOATUAL, NULL, 'S', 'N') AS IDINSTPEND,
      CASE
          WHEN UCX.NRVERSAOATUAL IS NULL AND EV.IDSTEVENTO IS NULL THEN
              'INSTALACAO_PENDENTE'
          WHEN EV.IDSTEVENTO IS NULL THEN
              'INSTALADO'
          ELSE
              EV.IDSTEVENTO
      END AS IDSTEVENTO,
      NVL(UCX.NRVERSAOATUAL, '-') AS NRVERSAOATUAL
    FROM
      UPDATERPRODFILICX UCX
      INNER JOIN FILIAL  F ON F.CDFILIAL = UCX.CDFILIAL
      LEFT JOIN ENDEFILI E ON E.CDFILIAL = F.CDFILIAL
                          AND E.IDTPENDEFILI = 'P'
      INNER JOIN CAIXA   C ON C.CDFILIAL = F.CDFILIAL
                          AND C.CDCAIXA = UCX.CDCAIXA
      INNER JOIN UPDATERPRODUTOS UP ON UCX.UPDATERPRODID = UP.ID
      LEFT JOIN UPDATEREVENTOSCX   EV ON EV.CDPRODUCTID    = UP.ID
                                  AND EV.CDFILIAL = F.CDFILIAL
                                  AND EV.CDCAIXA  = C.CDCAIXA
                                  AND EV.IDSTEVENTO NOT IN ( 'FINALIZADO', 'CANCELADO' )
    WHERE
          UCX.CDFILIAL = :CDFILIAL
      AND UCX.UPDATERPRODID = :ID
    ORDER BY
      UCX.CDCAIXA ASC,
      UCX.ID ASC
      `;            
      const params = {
        ID: PRODUCT_ID,
        CDFILIAL: CDFILIAL
      }      
      return OracleService.fetchAssoc(sql, params);  
  }

  insertUpdaterProdutos(params) {
    const sql = `
      INSERT INTO UPDATERPRODUTOS
                ( ID,  NMPRODUCT,  IDUTMULTVER, DTINCLUSAO, DTULTATU)
         VALUES (:ID, :NMPRODUCT, :IDUTMULTVER, SYSDATE,    SYSDATE)
    `;
    return OracleService.execute(sql, params);
  }

  updateUpdaterProdutos(params) {
    const sql = `
      UPDATE UPDATERPRODUTOS
         SET NMPRODUCT   = :NMPRODUCT,
             IDUTMULTVER = :IDUTMULTVER,
             IDATIVO     = 'S',
             DTULTATU    = SYSDATE
       WHERE ID = :ID
    `;
    return OracleService.execute(sql, params);
  }

  inactiveUpdaterProdutos(id) {
    const sql = `
      UPDATE UPDATERPRODUTOS
         SET IDATIVO  = 'N',
             DTULTATU = SYSDATE
       WHERE ID = :ID
    `;

    const params = {
      ID: id
    }

    return OracleService.execute(sql, params);
  }

  async getUpdaterProductsWithVersion(P_CDPRODUCTID) {
    const sql = `
    SELECT
        P.ID,
        P.NMPRODUCT,
        F.CDFILIAL,
        F.NMFILIAL,
        E.SGESTADO,
        LISTAGG(NVL(PF.NRVERSAOATUAL, '-'), ';') WITHIN GROUP (ORDER BY PF.ID) AS VERSOES,
        LISTAGG(PF.ID, ';') WITHIN GROUP (ORDER BY PF.ID) AS PRODFILIIDS,
        MAX(EV.IDSTEVENTO) AS IDSTEVENTO
    FROM
        UPDATERPRODFILI PF
        INNER JOIN UPDATERPRODUTOS P 
            ON ( PF.UPDATERPRODID = P.ID )
        INNER JOIN FILIAL F
            ON ( PF.CDFILIAL = F.CDFILIAL )
        LEFT JOIN ENDEFILI E 
            ON ( F.CDFILIAL = E.CDFILIAL )
            AND ( E.IDTPENDEFILI = 'P' )
        LEFT JOIN UPDATEREVENTOS EV
            ON ( P.ID = EV.CDPRODUCTID ) 
            AND ( PF.ID = EV.CDPRODFILIID )
            AND ( PF.CDFILIAL = EV.CDFILIAL )
            AND ( EV.IDSTEVENTO NOT IN ('FINALIZADO', 'CANCELADO', 'INSTALACAO_PENDENTE') )
    WHERE
        P.ID = :P_CDPRODUCTID
    GROUP BY
        P.ID,
        P.NMPRODUCT,
        F.CDFILIAL,
        F.NMFILIAL,
        E.SGESTADO
    `;
    const params = { P_CDPRODUCTID };
    return OracleService.fetchAssoc(sql, params);
  }
}

module.exports = new updaterProdutosModel();
